"""Ce module définit la classe Cercle utilisable avec pycairo,
l'interface Cairo de python.

"""


import math


class Cercle:
    """Classe permettant de définir et dessiner des cercles

    """

    def distance(xy0, xy1):
        """Méthode statique de calcul de distances entre deux
        points du plan.

        Arguments :
            xy0 : tuple d'entiers des coordonnées du premier point
            xy1 : tuple d'entiers des coordonnées du deuxième point

        """
        return math.sqrt(sum((b - a) ** 2 for a, b in zip(xy0, xy1)))

    def __init__(self, centre, rayon):
        """Constructeur

        Arguments :
            centre : tuple de float des coordonnées du centre du cercle
            rayon : entier définissant le rayon du cercle en pixels

        """
        self.centre = centre
        self.rayon = rayon

    def appartient_disque(self, point):
        """Vérifie qu'un point appartient au disque défini par le
        cercle

        Argument :
            point : tuple d'entiers des coordonnées du point à tester

        Renvoi : booléen

        """
        return Cercle.distance(self.centre, point) <= self.rayon

    def trace_cairo(self, ctx, couleur):
        """Trace un cercle dans un contexte Cairo avec la couleur donné

        Arguments :
            ctx : objet Context Cairo sur lequel le dessin sera à tracer
            couleur : tuple de float RGB correspondant à la couleur de tracé
                      chaque canal prend une valeur comprise entre 0 et 1

        """
        ctx.save()
        x_centre, y_centre = self.centre
        ctx.arc(x_centre, y_centre, self.rayon, 0, 2 * math.pi)
        rouge, vert, bleu = couleur
        ctx.set_source_rgb(rouge, vert, bleu)
        ctx.stroke()
        ctx.restore()
