"""Ce module définit la classe Fond de création d'une image
destinée à servir de fond d'écran.

Elle utilise la classe Cercle et permet de construire une image
faite de cercles qui ne se chevauchent pas.

La surface de dessin est gérée avec pycairo.

"""


import random
import cairo
import cercle_cairo as cercle


class Fond:
    """Cette classe permet de créer une image considérée comme un
    fond d'écran.

    Chaque fond peut être sauvegardée en format PNG.

    """

    def __init__(self, largeur=1920, hauteur=1080, couleur=(0, 0, 0)):
        """Constructeur

        Arguments :
            largeur (optionnel) : largeur de l'image (par défaut celle de FHD)
            hauteur (optionnel) : hauteur de l'image (par défaut celle de FHD)
            couleur (optionnel) : tuple de float RGB correspondant à la couleur
                                  de tracé, chaque canal prend une valeur
                                  comprise entre 0 et 1.
                                  Par défaut (0, 0, 0) : noir

        """
        self.largeur = largeur
        self.hauteur = hauteur
        self.surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                          self.largeur, self.hauteur)
        self.ctx = cairo.Context(self.surface)
        self.ctx.set_source_rgb(*couleur)
        self.ctx.paint()
        self.cercles = []

    def est_dans_un_cercle(self, point):
        """Vérifie qu'un point donné par ses coordonnées appartient
        à l'un des cercles déjà construit.

        Arguments :
            point : tuple d'entiers des coordonnées du point à tester

        Renvoi : booléen

        """
        dedans = False
        i = 0

        while i < len(self.cercles) and ...
            un_cercle = self.cercles[i]
            dedans = ...
            i += 1

        return dedans

    def rayon_max_bords(self, position):
        """Renvoie la distance la plus courte des quatre bords~: on
        s'assure que le cercle soit intégralement visible.

        Arguments :
            position : tuple d'entiers des coordonnées du centre du cercle

        Renvoi : entier

        """
        dist_gauche = position[0]
        dist_droite = self.largeur - 1 - position[0]
        dist_bas = ...
        dist_haut = ...

        # utiliser une fonction qui permet de calculer le minimum
        # des 4 valeurs précédentes
        return ...

    def rayon_max_cercles(self, position):
        """Détermine le rayon maximal du cercle possible à la
        position donnée en évitant les chevauchements avec ce qui
        existe déjà et en s'assurant que le cercle soit
        intégralement visible.

        Arguments :
            position : tuple d'entiers des coordonnées du centre du cercle

        Renvoi : entier

        """
        r_max = float('inf')
        for c in ...:
            r_max = min(r_max, ...)
        return r_max

    def tracer_cercles(self, effectif, couleur, max_admis=300):
        """Construit les cercles en nombre demandé. Les cercles ne
        se chevauchent pas et sont intégralement tracés dans
        l'image.

        Si trop de cercles sont demandés, l'algorithme choisi ne
        permet pas d'assurer une fin rapide, voire une fin tout
        court...

        Arguments :
            nb : nombre de cercles à tracer
            couleur : tuple de float RGB correspondant à la couleur de tracé
                      chaque canal prend une valeur comprise entre 0 et 1
            max_admis (optionnel) : entier égal au rayon maximal qu'on ne
                                    veut pas dépasser, par défaut 300 px

        Renvoi : None

        """
        for _ in range(effectif):

            position_valide = ...
            while not position_valide:
                x, y = ...
                position_valide = not self.est_dans_un_cercle((x, y))

            position = (x, y)
            rayon = min(...,
                        ...,
                        ...)
            nouveau_cercle = ...
            nouveau_cercle.trace_cairo(..., ...)
            self.cercles.append(nouveau_cercle)

    def sauvegarder(self, nom_fichier_png):
        """Permet l'enregistrement de l'image dans un fichier PNG.

        Argument :
            nom_fichier_png : chaîne de caractères du nom de fichier,
                              préciser l'extension ".png" dans le nom

        """
        self.surface.write_to_png(nom_fichier_png)
